from __future__ import annotations

from typing import TYPE_CHECKING, ClassVar, override

from pydantic_settings_sops import SOPSConfigSettingsSource

from utilities.iterables import always_iterable
from utilities.pydantic_settings import CustomBaseSettings

if TYPE_CHECKING:
    from collections.abc import Iterator

    from pydantic_settings import BaseSettings, PydanticBaseSettingsSource

    from utilities.types import MaybeIterable, PathLike


class SopsBaseSettings(CustomBaseSettings):
    """Base settings for loading secrets using `sops/age`."""

    # paths
    secret_files: ClassVar[MaybeIterable[PathLike]] = ()

    @classmethod
    @override
    def _yield_base_settings_sources(
        cls,
        settings_cls: type[BaseSettings],
        env_settings: PydanticBaseSettingsSource,
        /,
    ) -> Iterator[PydanticBaseSettingsSource]:
        yield from super()._yield_base_settings_sources(settings_cls, env_settings)
        for file in always_iterable(cls.secret_files):
            yield SOPSConfigSettingsSource(
                settings_cls,  # pyright: ignore[reportArgumentType],
                json_file=file,
            )


__all__ = ["SopsBaseSettings"]



from __future__ import annotations

from json import dumps
from typing import TYPE_CHECKING, ClassVar

from pydantic_settings import BaseSettings, SettingsConfigDict

from utilities.os import temp_environ
from utilities.pydantic_settings import CustomBaseSettings, load_settings

if TYPE_CHECKING:
    from pathlib import Path

    from utilities.types import MaybeIterable, PathLike


class TestCustomBaseSettings:
    def test_main(self, *, tmp_path: Path) -> None:
        file = tmp_path.joinpath("settings.toml")
        _ = file.write_text(dumps({"x": 1, "y": 2}))

        class Settings(CustomBaseSettings):
            json_files: ClassVar[MaybeIterable[PathLike]] = file
            x: int
            y: int

        settings = load_settings(Settings)
        assert settings.x == 1
        assert settings.y == 2

    def test_env_var(self) -> None:
        class Settings(CustomBaseSettings):
            x: int

        with temp_environ(x="1"):
            settings = load_settings(Settings)
        assert settings.x == 1

    def test_env_var_with_prefix(self) -> None:
        class Settings(CustomBaseSettings):
            model_config = SettingsConfigDict(env_prefix="test_")
            x: int

        with temp_environ(test_x="1"):
            settings = load_settings(Settings)
        assert settings.x == 1

    def test_env_var_with_nested(self) -> None:
        class Settings(CustomBaseSettings):
            inner: Inner

        class Inner(BaseSettings):
            x: int

        _ = Settings.model_rebuild()

        with temp_environ(inner__x="1"):
            settings = load_settings(Settings)
        assert settings.inner.x == 1

    def test_env_var_with_prefix_and_nested(self) -> None:
        class Settings(CustomBaseSettings):
            model_config = SettingsConfigDict(env_prefix="test__")
            inner: Inner

        class Inner(BaseSettings):
            x: int

        _ = Settings.model_rebuild()
        with temp_environ(test__inner__x="1"):
            settings = load_settings(Settings)
        assert settings.inner.x == 1

    def test_no_files(self) -> None:
        class Settings(CustomBaseSettings): ...

        _ = load_settings(Settings)



from __future__ import annotations

from typing import TYPE_CHECKING, ClassVar, override

from pydantic_settings import (
    BaseSettings,
    JsonConfigSettingsSource,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)

from utilities.iterables import always_iterable

if TYPE_CHECKING:
    from collections.abc import Iterator

    from utilities.types import MaybeIterable, PathLike


class CustomBaseSettings(BaseSettings):
    """Base settings for loading JSON files."""

    # paths
    json_files: ClassVar[MaybeIterable[PathLike]] = ()

    # config
    model_config: ClassVar[SettingsConfigDict] = SettingsConfigDict(
        env_nested_delimiter="__"
    )

    @classmethod
    @override
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        _ = (init_settings, dotenv_settings, file_secret_settings)
        return tuple(cls._yield_base_settings_sources(settings_cls, env_settings))

    @classmethod
    def _yield_base_settings_sources(
        cls,
        settings_cls: type[BaseSettings],
        env_settings: PydanticBaseSettingsSource,
        /,
    ) -> Iterator[PydanticBaseSettingsSource]:
        yield env_settings
        for file in always_iterable(cls.json_files):
            yield JsonConfigSettingsSource(settings_cls, json_file=file)


def load_settings[T: BaseSettings](cls: type[T], /) -> T:
    """Load a set of settings."""
    return cls()


__all__ = ["CustomBaseSettings", "load_settings"]
